@@ -7,7 +7,7 @@ Attean::API::Iterator - Typed iterator
 
 =head1 VERSION
 
-This document describes Attean::API::Iterator version 0.010
+This document describes Attean::API::Iterator version 0.011
 
 =head1 DESCRIPTION
 
@@ -94,7 +94,7 @@ elements from the referent.
 
 =cut
 
-package Attean::API::Iterator 0.010 {
+package Attean::API::Iterator 0.011 {
 	use Moo::Role;
 	use Scalar::Util qw(blessed);
 	use Types::Standard qw(Str Object InstanceOf);
@@ -254,7 +254,7 @@ Print each item as it is consumed (with the string generated by C<< as_string >>
 	}
 }
 
-package Attean::API::RepeatableIterator 0.010 {
+package Attean::API::RepeatableIterator 0.011 {
 	use Moo::Role;
 	requires 'reset';
 	
@@ -290,7 +290,7 @@ package Attean::API::CanonicalizingBindingIterator {
 	}
 }
 
-package Attean::API::ResultOrTermIterator 0.010 {
+package Attean::API::ResultOrTermIterator 0.011 {
 	use Moo::Role;
 	use Types::Standard qw(ArrayRef Str);
 	has 'variables' => (is => 'rw', isa => ArrayRef[Str], default => sub { [] });
@@ -307,9 +307,28 @@ package Attean::API::ResultOrTermIterator 0.010 {
 			}
 		});
 	}
+	
+	around 'grep' => sub {
+		my $orig	= shift;
+		my $self	= shift;
+		my $block	= shift;
+		my $iter	= $orig->($self, $block, @_);
+		Attean::CodeIterator->new(
+			item_type => $iter->item_type,
+			generator => sub {
+				while (1) {
+					my $item	= $iter->next();
+					return unless defined($item);
+					local($_)	= $item;
+					return $item if ($block->($item));
+				}
+			},
+			variables => $self->variables,
+		);
+	};
 }
 
-package Attean::API::TripleIterator 0.010 {
+package Attean::API::TripleIterator 0.011 {
 	use Moo::Role;
 	with 'Attean::API::CanonicalizingBindingIterator';
 	sub as_quads {
@@ -319,12 +338,12 @@ package Attean::API::TripleIterator 0.010 {
 	}
 }
 
-package Attean::API::QuadIterator 0.010 {
+package Attean::API::QuadIterator 0.011 {
 	use Moo::Role;
 	with 'Attean::API::CanonicalizingBindingIterator';
 }
 
-package Attean::API::MixedStatementIterator 0.010 {
+package Attean::API::MixedStatementIterator 0.011 {
 	use Moo::Role;
 	with 'Attean::API::CanonicalizingBindingIterator';
 	sub as_quads {
@@ -337,7 +356,7 @@ package Attean::API::MixedStatementIterator 0.010 {
 	}
 }
 
-package Attean::API::ResultIterator 0.010 {
+package Attean::API::ResultIterator 0.011 {
 	use Moo::Role;
 	use Types::Standard qw(Str ArrayRef);
 	with 'Attean::API::CanonicalizingBindingIterator';
@@ -358,7 +377,7 @@ package Attean::API::ResultIterator 0.010 {
 	}
 }
 
-package Attean::API::TermIterator 0.010 {
+package Attean::API::TermIterator 0.011 {
 	use Moo::Role;
 	sub canonicalize {
 		my $self	= shift;
