@@ -7,7 +7,7 @@ Attean::API::Store - Triple/quad store role
 
 =head1 VERSION
 
-This document describes Attean::Store version 0.010
+This document describes Attean::Store version 0.011
 
 =head1 DESCRIPTION
 
@@ -35,16 +35,26 @@ The Attean::Store role is an empty role that more specialized roles conform to:
 
 =cut
 
-package Attean::API::Store 0.010 {
+package Attean::API::Store 0.011 {
 	use Moo::Role;
 }
 
-package Attean::API::TripleStore 0.010 {
+package Attean::API::TripleStore 0.011 {
 	use Moo::Role;
+	use Scalar::Util qw(blessed);
+	use namespace::clean;
+	
 	with 'Attean::API::Store';
 
 	requires 'get_triples';
 
+	before 'get_triples' => sub {
+		if (scalar(@_) == 2 and blessed($_[1]) and not($_[1]->does('Attean::API::TermOrVariable'))) {
+			my $type	= ref($_[0]);
+			die "get_triples called with a single $type argument, but expecting a list of terms/variables";
+		}
+	};
+	
 	sub count_triples {
 		my $self	= shift;
 		my $iter	= $self->get_triples(@_);
@@ -66,7 +76,7 @@ package Attean::API::TripleStore 0.010 {
 	}
 }
 
-package Attean::API::MutableTripleStore 0.010 {
+package Attean::API::MutableTripleStore 0.011 {
 	use Moo::Role;
 	with 'Attean::API::TripleStore';
 	
@@ -74,26 +84,36 @@ package Attean::API::MutableTripleStore 0.010 {
 	requires 'remove_triple';
 }
 
-package Attean::API::ETagCacheableTripleStore 0.010 {
+package Attean::API::ETagCacheableTripleStore 0.011 {
 	use Moo::Role;
 	with 'Attean::API::TripleStore';
 	
 	requires 'etag_value_for_triples';
 }
 
-package Attean::API::TimeCacheableTripleStore 0.010 {
+package Attean::API::TimeCacheableTripleStore 0.011 {
 	use Moo::Role;
 	with 'Attean::API::TripleStore';
 	
 	requires 'mtime_for_triples';
 }
 
-package Attean::API::QuadStore 0.010 {
+package Attean::API::QuadStore 0.011 {
 	use Moo::Role;
+	use Scalar::Util qw(blessed);
+	use namespace::clean;
+	
 	with 'Attean::API::Store';
 
 	requires 'get_quads';
 
+	before 'get_quads' => sub {
+		if (scalar(@_) == 2 and blessed($_[1]) and not($_[1]->does('Attean::API::TermOrVariable'))) {
+			my $type	= ref($_[0]);
+			die "get_quads called with a single $type argument, but expecting a list of terms/variables";
+		}
+	};
+	
 	sub count_quads {
 		my $self	= shift;
 		my $iter	= $self->get_quads(@_);
@@ -126,7 +146,7 @@ package Attean::API::QuadStore 0.010 {
 	}
 }
 
-package Attean::API::MutableQuadStore 0.010 {
+package Attean::API::MutableQuadStore 0.011 {
 	use Moo::Role;
 	use Type::Tiny::Role;
 	with 'Attean::API::QuadStore';
@@ -148,14 +168,14 @@ package Attean::API::MutableQuadStore 0.010 {
 	}
 }
 
-package Attean::API::ETagCacheableQuadStore 0.010 {
+package Attean::API::ETagCacheableQuadStore 0.011 {
 	use Moo::Role;
 	with 'Attean::API::QuadStore';
 	
 	requires 'etag_value_for_quads';
 }
 
-package Attean::API::TimeCacheableQuadStore 0.010 {
+package Attean::API::TimeCacheableQuadStore 0.011 {
 	use Moo::Role;
 	with 'Attean::API::QuadStore';
 	
