@@ -7,7 +7,7 @@ Attean::Plan - Representation of SPARQL query plan operators
 
 =head1 VERSION
 
-This document describes Attean::Plan version 0.010
+This document describes Attean::Plan version 0.011
 
 =head1 SYNOPSIS
 
@@ -31,7 +31,7 @@ Evaluates a quad pattern against the model.
 
 =cut
 
-package Attean::Plan::Quad 0.010 {
+package Attean::Plan::Quad 0.011 {
 	use Moo;
 	use Scalar::Util qw(blessed reftype);
 	use Types::Standard qw(ConsumerOf ArrayRef);
@@ -136,7 +136,7 @@ Evaluates a join (natural-, anti-, or left-) using a nested loop.
 
 =cut
 
-package Attean::Plan::NestedLoopJoin 0.010 {
+package Attean::Plan::NestedLoopJoin 0.011 {
 	use Moo;
 	use List::MoreUtils qw(all);
 	use namespace::clean;
@@ -179,6 +179,7 @@ package Attean::Plan::NestedLoopJoin 0.010 {
 		my @children	= @_;
 		my $left		= $self->left;
 		my $anti		= $self->anti;
+		my $iter_variables	= $self->in_scope_variables;
 		
 		return sub {
 			my ($lhs, $rhs)	= map { $_->() } @children;
@@ -211,7 +212,8 @@ package Attean::Plan::NestedLoopJoin 0.010 {
 			}
 			return Attean::ListIterator->new(
 				item_type => 'Attean::API::Result',
-				values => \@results
+				variables => $iter_variables,
+				values => \@results,
 			);
 		}
 	}
@@ -223,7 +225,7 @@ Evaluates a join (natural-, anti-, or left-) using a hash join.
 
 =cut
 
-package Attean::Plan::HashJoin 0.010 {
+package Attean::Plan::HashJoin 0.011 {
 	use Moo;
 	use List::MoreUtils qw(all);
 	use namespace::clean;
@@ -273,6 +275,8 @@ package Attean::Plan::HashJoin 0.010 {
 		my $model		= shift;
 		my @children	= @_;
 		my $left	= $self->left;
+		my $iter_variables	= $self->in_scope_variables;
+
 		return sub {
 			my %hash;
 			my @vars	= @{ $self->join_variables };
@@ -351,6 +355,7 @@ package Attean::Plan::HashJoin 0.010 {
 			}
 			return Attean::ListIterator->new(
 				item_type => 'Attean::API::Result',
+				variables => $iter_variables,
 				values => \@results
 			);
 		}
@@ -361,7 +366,7 @@ package Attean::Plan::HashJoin 0.010 {
 
 =cut
 
-package Attean::Plan::Construct 0.010 {
+package Attean::Plan::Construct 0.011 {
 	use Moo;
 	use List::MoreUtils qw(all);
 	use Types::Standard qw(Str ArrayRef ConsumerOf InstanceOf);
@@ -406,6 +411,7 @@ package Attean::Plan::Construct 0.010 {
 			die "Plan children do not all consume BindingSubstitutionPlan role:\n" . $self->as_string;
 		}
 		
+		warn "TODO: fix substitute_impl to substitute construct triples";
 		my @children	= map { $_->substitute_impl($model, $b) } @{ $self->children };
 		return $self->_impl($model, @children);
 	}
@@ -447,6 +453,89 @@ package Attean::Plan::Construct 0.010 {
 	}
 }
 
+=item * L<Attean::Plan::Describe>
+
+=cut
+
+package Attean::Plan::Describe 0.011 {
+	use Moo;
+	use Attean::RDF;
+	use List::MoreUtils qw(all);
+	use Types::Standard qw(Str ArrayRef ConsumerOf InstanceOf);
+	use namespace::clean;
+
+	has 'graph'	=> (is => 'ro');
+	has 'terms' => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::TermOrVariable']]);
+
+	with 'Attean::API::BindingSubstitutionPlan', 'Attean::API::UnaryQueryTree';
+	with 'Attean::API::UnionScopeVariablesPlan';
+
+	sub plan_as_string {
+		my $self	= shift;
+		my $terms	= $self->terms;
+		return sprintf('Describe { %s }', join(' . ', map { $_->as_string } @$terms));
+	}
+
+	sub impl {
+		my $self	= shift;
+		my $model	= shift;
+		my @children	= map { $_->impl($model) } @{ $self->children };
+		return $self->_impl($model, @children);
+	}
+	
+	sub substitute_impl {
+		my $self	= shift;
+		my $model	= shift;
+		my $b		= shift;
+		unless (all { $_->does('Attean::API::BindingSubstitutionPlan') } @{ $self->children }) {
+			die "Plan children do not all consume BindingSubstitutionPlan role:\n" . $self->as_string;
+		}
+		
+		warn "TODO: fix substitute_impl to substitute describe terms";
+		my @children	= map { $_->substitute_impl($model, $b) } @{ $self->children };
+		return $self->_impl($model, @children);
+	}
+	
+	sub _impl {
+		my $self		= shift;
+		my $model		= shift;
+		my $child		= shift;
+		
+		my $graph		= $self->graph;
+		my @terms		= @{ $self->terms };
+		# TODO: Split @terms into ground terms and variables.
+		#       Only call get_quads once for ground terms.
+		#       For variable terms, call get_quads for each variable-result combination.
+		return sub {
+			my $iter	= $child->();
+			my @buffer;
+			my %seen;
+			return Attean::CodeIterator->new(
+				item_type => 'Attean::API::Triple',
+				generator => sub {
+					if (scalar(@buffer)) {
+						return shift(@buffer);
+					}
+					while (my $row = $iter->next) {
+						foreach my $term (@terms) {
+							my $value	= $term->apply_binding($row);
+							if ($value->does('Attean::API::Term')) {
+								my $iter	= $model->get_quads( $value, variable('predicate'), variable('object'), $graph );
+								push(@buffer, $iter->elements);
+							}
+							if (scalar(@buffer)) {
+								return shift(@buffer);
+							}
+						}
+					}
+				}
+			)->grep(sub {
+				return not $seen{$_->as_string}++;
+			});
+		}
+	}
+}
+
 =item * L<Attean::Plan::EBVFilter>
 
 Filters results from a sub-plan based on the effective boolean value of a
@@ -454,7 +543,7 @@ named variable binding.
 
 =cut
 
-package Attean::Plan::EBVFilter 0.010 {
+package Attean::Plan::EBVFilter 0.011 {
 	use Moo;
 	use Scalar::Util qw(blessed);
 	use Types::Standard qw(Str ConsumerOf);
@@ -513,7 +602,7 @@ ordering.
 
 =cut
 
-package Attean::Plan::Merge 0.010 {
+package Attean::Plan::Merge 0.011 {
 	use Moo;
 	use Scalar::Util qw(blessed);
 	use Types::Standard qw(Str ArrayRef ConsumerOf);
@@ -542,7 +631,7 @@ Evaluates a set of sub-plans, returning the union of results.
 
 =cut
 
-package Attean::Plan::Union 0.010 {
+package Attean::Plan::Union 0.011 {
 	use Moo;
 	use Scalar::Util qw(blessed);
 	use namespace::clean;
@@ -575,11 +664,14 @@ package Attean::Plan::Union 0.010 {
 		my $self		= shift;
 		my $model		= shift;
 		my @children	= @_;
+		my $iter_variables	= $self->in_scope_variables;
+
 		return sub {
 			my $current	= shift(@children);
 			my $iter	= $current->();
 			return Attean::CodeIterator->new(
 				item_type => 'Attean::API::Result',
+				variables => $iter_variables,
 				generator => sub {
 					while (blessed($iter)) {
 						my $row	= $iter->next();
@@ -607,7 +699,7 @@ expressions, binding the produced values to new variables.
 
 =cut
 
-package Attean::Plan::Extend 0.010 {
+package Attean::Plan::Extend 0.011 {
 	use Moo;
 	use Encode;
 	use Data::UUID;
@@ -1160,10 +1252,13 @@ package Attean::Plan::Extend 0.010 {
 		my $model	= shift;
 		my $impl	= shift;
 		my %exprs	= @_;
+		my $iter_variables	= $self->in_scope_variables;
+
 		return sub {
 			my $iter	= $impl->();
 			return Attean::CodeIterator->new(
 				item_type => 'Attean::API::Result',
+				variables => $iter_variables,
 				generator => sub {
 					ROW: while (my $r = $iter->next) {
 # 						warn 'Extend Row -------------------------------> ' . $r->as_string;
@@ -1198,7 +1293,7 @@ hash of already-seen results.
 
 =cut
 
-package Attean::Plan::HashDistinct 0.010 {
+package Attean::Plan::HashDistinct 0.011 {
 	use Moo;
 	use namespace::clean;
 	
@@ -1226,7 +1321,7 @@ filtering out sequential duplicates.
 
 =cut
 
-package Attean::Plan::Unique 0.010 {
+package Attean::Plan::Unique 0.011 {
 	use Moo;
 	use namespace::clean;
 	
@@ -1261,7 +1356,7 @@ number of results ("offset") and limiting the total number of returned results
 
 =cut
 
-package Attean::Plan::Slice 0.010 {
+package Attean::Plan::Slice 0.011 {
 	use Moo;
 	use Types::Standard qw(Int);
 	use namespace::clean;
@@ -1302,7 +1397,7 @@ of variable bindings in each result.
 
 =cut
 
-package Attean::Plan::Project 0.010 {
+package Attean::Plan::Project 0.011 {
 	use Moo;
 	with 'Attean::API::BindingSubstitutionPlan', 'Attean::API::UnaryQueryTree';
 	use Types::Standard qw(ArrayRef ConsumerOf);
@@ -1341,6 +1436,7 @@ package Attean::Plan::Project 0.010 {
 		my $bind	= shift;
 		my ($impl)	= map { $_->substitute_impl($model, $bind) } @{ $self->children };
 		my @vars	= map { $_->value } @{ $self->variables };
+		my $iter_variables	= $self->in_scope_variables;
 
 		# TODO: substitute variables in the projection where appropriate
 		return sub {
@@ -1349,7 +1445,7 @@ package Attean::Plan::Project 0.010 {
 				my $r	= shift;
 				my $b	= { map { my $t	= $r->value($_); $t	? ($_ => $t) : () } @vars };
 				return Attean::Result->new( bindings => $b );
-			});
+			}, $iter->item_type, $iter_variables);
 		};
 	}
 	
@@ -1358,6 +1454,7 @@ package Attean::Plan::Project 0.010 {
 		my $model	= shift;
 		my ($impl)	= map { $_->impl($model) } @{ $self->children };
 		my @vars	= map { $_->value } @{ $self->variables };
+		my $iter_variables	= $self->in_scope_variables;
 
 		return sub {
 			my $iter	= $impl->();
@@ -1365,7 +1462,7 @@ package Attean::Plan::Project 0.010 {
 				my $r	= shift;
 				my $b	= { map { my $t	= $r->value($_); $t	? ($_ => $t) : () } @vars };
 				return Attean::Result->new( bindings => $b );
-			});
+			}, $iter->item_type, $iter_variables);
 		};
 	}
 }
@@ -1377,7 +1474,7 @@ sorting is applied.
 
 =cut
 
-package Attean::Plan::OrderBy 0.010 {
+package Attean::Plan::OrderBy 0.011 {
 	use Moo;
 	use Types::Standard qw(HashRef ArrayRef InstanceOf Bool Str);
 	use namespace::clean;
@@ -1426,11 +1523,17 @@ package Attean::Plan::OrderBy 0.010 {
 		my $vars	= $self->variables;
 		my $ascending	= $self->ascending;
 		my ($impl)	= map { $_->impl($model) } @{ $self->children };
+		my $iter_variables	= $self->in_scope_variables;
+
 		return sub {
 			my $iter	= $impl->();
 			my @rows	= $iter->elements;
 			my @sorted	= $self->sort_rows($vars, $ascending, \@rows);
-			return Attean::ListIterator->new( values => \@sorted, item_type => $iter->item_type);
+			return Attean::ListIterator->new(
+				values => \@sorted,
+				variables => $iter_variables,
+				item_type => $iter->item_type
+			);
 		}
 	}
 }
@@ -1441,7 +1544,7 @@ Evaluates a SPARQL query against a remove endpoint.
 
 =cut
 
-package Attean::Plan::Service 0.010 {
+package Attean::Plan::Service 0.011 {
 	use Moo;
 	use Types::Standard qw(ConsumerOf Bool Str);
 	use namespace::clean;
@@ -1473,7 +1576,7 @@ Returns a constant set of results.
 
 =cut
 
-package Attean::Plan::Table 0.010 {
+package Attean::Plan::Table 0.011 {
 	use Moo;
 	use Types::Standard qw(ArrayRef ConsumerOf);
 	use namespace::clean;
@@ -1513,9 +1616,12 @@ package Attean::Plan::Table 0.010 {
 		my $self	= shift;
 		my $model	= shift;
 		my $rows	= $self->rows;
+		my $iter_variables	= $self->in_scope_variables;
+
 		return sub {
 			return Attean::ListIterator->new(
 				item_type => 'Attean::API::Result',
+				variables => $iter_variables,
 				values => $rows
 			);
 		};
@@ -1526,7 +1632,7 @@ package Attean::Plan::Table 0.010 {
 
 =cut
 
-package Attean::Plan::ALPPath 0.010 {
+package Attean::Plan::ALPPath 0.011 {
 	use Moo;
 	use Attean::TreeRewriter;
 	use Types::Standard qw(ArrayRef ConsumerOf);
@@ -1609,7 +1715,8 @@ package Attean::Plan::ALPPath 0.010 {
 		my $start	= $self->step_begin->value;
 		my $end		= $self->step_end->value;
 		my $skip	= $self->skip;
-		
+		my $iter_variables	= $self->in_scope_variables;
+
 		for ($subject, $object) {
 			if ($_->does('Attean::API::Variable')) {
 				my $name	= $_->value;
@@ -1638,6 +1745,7 @@ package Attean::Plan::ALPPath 0.010 {
 				}
 				return Attean::ListIterator->new(
 					item_type => 'Attean::API::Result',
+					variables => $iter_variables,
 					values => \@rows,
 				);
 			};
@@ -1648,6 +1756,7 @@ package Attean::Plan::ALPPath 0.010 {
 				my @rows	= map { Attean::Result->new( bindings => { $object->value => $_ } ) } (values %seen);
 				return Attean::ListIterator->new(
 					item_type => 'Attean::API::Result',
+					variables => $iter_variables,
 					values => \@rows,
 				);
 			};
@@ -1658,16 +1767,24 @@ package Attean::Plan::ALPPath 0.010 {
 				my %seen;
 				alp($model, $graph, $skip, $subject, $path, \%seen, $start, $end, $bind);
 				if (exists $seen{ $object->as_string }) {
-					return Attean::ListIterator->new( item_type => 'Attean::API::Result', values => [Attean::Result->new()] );
+					return Attean::ListIterator->new(
+						item_type => 'Attean::API::Result',
+						variables => $iter_variables,
+						values => [Attean::Result->new()]
+					);
 				} else {
-					return Attean::ListIterator->new( item_type => 'Attean::API::Result', values => [] );
+					return Attean::ListIterator->new(
+						item_type => 'Attean::API::Result',
+						variables => $iter_variables,
+						values => []
+					);
 				}
 			};
 		}
 	}
 }
 
-package Attean::Plan::ZeroOrOnePath 0.010 {
+package Attean::Plan::ZeroOrOnePath 0.011 {
 	use Moo;
 	use Attean::TreeRewriter;
 	use Types::Standard qw(ArrayRef ConsumerOf);
@@ -1702,6 +1819,7 @@ package Attean::Plan::ZeroOrOnePath 0.010 {
 		my $model	= shift;
 		my $bind	= shift;
 		my ($impl)	= map { $_->substitute_impl($model, $bind) } @{ $self->children };
+		my $iter_variables	= $self->in_scope_variables;
 
 		my $subject	= $self->subject;
 		my $object	= $self->object;
@@ -1737,6 +1855,7 @@ package Attean::Plan::ZeroOrOnePath 0.010 {
 			my %seen;
 			return Attean::CodeIterator->new(
 				item_type => 'Attean::API::Result',
+				variables => $iter_variables,
 				generator => sub {
 					while (scalar(@extra)) {
 						my $r	= shift(@extra);
@@ -1764,7 +1883,7 @@ results were produced by evaluating the sub-plan.
 
 =cut
 
-package Attean::Plan::Exists 0.010 {
+package Attean::Plan::Exists 0.011 {
 	use Moo;
 	use Types::Standard qw(ArrayRef ConsumerOf);
 	use namespace::clean;
@@ -1794,7 +1913,7 @@ package Attean::Plan::Exists 0.010 {
 
 =cut
 
-package Attean::Plan::Aggregate 0.010 {
+package Attean::Plan::Aggregate 0.011 {
 	use Moo;
 	use Encode;
 	use Data::UUID;
@@ -1846,7 +1965,15 @@ package Attean::Plan::Aggregate 0.010 {
 		if ($op eq 'COUNT') {
 			my $count	= 0;
 			foreach my $r (@$rows) {
-				$count++;
+				if ($e) {
+					my $term	= Attean::Plan::Extend->evaluate_expression($model, $e, $r);
+					if ($term) {
+						$count++;
+					}
+				} else {
+					# This is the special-case branch for COUNT(*)
+					$count++;
+				}
 			}
 			return Attean::Literal->new(value => $count, datatype => 'http://www.w3.org/2001/XMLSchema#integer');
 		} elsif ($op eq 'SUM') {
@@ -1918,6 +2045,8 @@ package Attean::Plan::Aggregate 0.010 {
 		my $model	= shift;
 		my %aggs	= %{ $self->aggregates };
 		my @groups	= @{ $self->groups };
+		my $iter_variables	= $self->in_scope_variables;
+
 		my $group_template_generator	= sub {
 			my $r	= shift;
 			my %components;
@@ -2005,7 +2134,11 @@ package Attean::Plan::Aggregate 0.010 {
 					push(@results, $result);
 				}
 			}
-			return Attean::ListIterator->new(values => \@results, item_type => 'Attean::API::Result');
+			return Attean::ListIterator->new(
+				values => \@results,
+				variables => $iter_variables,
+				item_type => 'Attean::API::Result'
+			);
 		};
 	}
 }
@@ -2014,7 +2147,7 @@ package Attean::Plan::Aggregate 0.010 {
 # 
 # =cut
 # 
-# package Attean::Algebra::Ask 0.010 {
+# package Attean::Algebra::Ask 0.011 {
 # 	use Moo;
 # 	sub in_scope_variables { return; }
 # 	with 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
@@ -2036,7 +2169,7 @@ package Attean::Plan::Aggregate 0.010 {
 # 
 # =cut
 # 
-# package Attean::Algebra::Construct 0.010 {
+# package Attean::Algebra::Construct 0.011 {
 # 	use Moo;
 # 	use Types::Standard qw(ArrayRef ConsumerOf);
 # 	has 'triples' => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::TriplePattern']]);
@@ -2063,7 +2196,7 @@ package Attean::Plan::Aggregate 0.010 {
 # 
 # =cut
 # 
-# package Attean::Algebra::Path 0.010 {
+# package Attean::Algebra::Path 0.011 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	sub in_scope_variables {
@@ -2101,7 +2234,7 @@ package Attean::Plan::Aggregate 0.010 {
 # 
 # =cut
 # 
-# package Attean::Algebra::NegatedPropertySet 0.010 {
+# package Attean::Algebra::NegatedPropertySet 0.011 {
 # 	use Moo;
 # 	use Types::Standard qw(ArrayRef ConsumerOf);
 # 	with 'Attean::API::PropertyPath';
@@ -2122,7 +2255,7 @@ package Attean::Plan::Aggregate 0.010 {
 # 
 # =cut
 # 
-# package Attean::Algebra::PredicatePath 0.010 {
+# package Attean::Algebra::PredicatePath 0.011 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	with 'Attean::API::PropertyPath';
@@ -2146,7 +2279,7 @@ package Attean::Plan::Aggregate 0.010 {
 # 
 # =cut
 # 
-# package Attean::Algebra::InversePath 0.010 {
+# package Attean::Algebra::InversePath 0.011 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	with 'Attean::API::UnaryPropertyPath';
@@ -2162,7 +2295,7 @@ package Attean::Plan::Aggregate 0.010 {
 # 
 # =cut
 # 
-# package Attean::Algebra::SequencePath 0.010 {
+# package Attean::Algebra::SequencePath 0.011 {
 # 	use Moo;
 # 	with 'Attean::API::NaryPropertyPath';
 # 	sub separator { return "/" }
@@ -2177,7 +2310,7 @@ package Attean::Plan::Aggregate 0.010 {
 # 
 # =cut
 # 
-# package Attean::Algebra::AlternativePath 0.010 {
+# package Attean::Algebra::AlternativePath 0.011 {
 # 	use Moo;
 # 	with 'Attean::API::NaryPropertyPath';
 # 	sub separator { return "|" }
@@ -2192,7 +2325,7 @@ package Attean::Plan::Aggregate 0.010 {
 # 
 # =cut
 # 
-# package Attean::Algebra::ZeroOrMorePath 0.010 {
+# package Attean::Algebra::ZeroOrMorePath 0.011 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	with 'Attean::API::UnaryPropertyPath';
@@ -2208,7 +2341,7 @@ package Attean::Plan::Aggregate 0.010 {
 # 
 # =cut
 # 
-# package Attean::Algebra::OneOrMorePath 0.010 {
+# package Attean::Algebra::OneOrMorePath 0.011 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	with 'Attean::API::UnaryPropertyPath';
@@ -2224,7 +2357,7 @@ package Attean::Plan::Aggregate 0.010 {
 # 
 # =cut
 # 
-# package Attean::Algebra::ZeroOrOnePath 0.010 {
+# package Attean::Algebra::ZeroOrOnePath 0.011 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	with 'Attean::API::UnaryPropertyPath';
