@@ -4,7 +4,7 @@ AtteanX::Parser::RDFXML - RDF/XML Parser
 
 =head1 VERSION
 
-This document describes AtteanX::Parser::RDFXML version 0.010
+This document describes AtteanX::Parser::RDFXML version 0.011
 
 =head1 SYNOPSIS
 
@@ -41,7 +41,7 @@ A string prefix for identifiers generated for blank nodes.
 use v5.14;
 use warnings;
 
-package AtteanX::Parser::RDFXML 0.010 {
+package AtteanX::Parser::RDFXML 0.011 {
 	use Moo;
 	use Types::Standard qw(Str Object);
 	use Attean;
@@ -118,6 +118,10 @@ the data read from the UTF-8 encoded byte string C<< $data >>.
 		if (my $map = $self->namespaces) {
 			push(@args, namespaces => $map);
 		}
+		
+		if ($self->has_base) {
+			push(@args, base => $self->base);
+		}
 		my $saxhandler	= AtteanX::Parser::RDFXML::SAXHandler->new( bnode_prefix => $self->bnode_prefix, handler => $self->handler, @args );
 		my $p			= XML::SAX::ParserFactory->parser(Handler => $saxhandler);
 		$saxhandler->push_base( $self->base ) if ($self->has_base);
@@ -125,12 +129,14 @@ the data read from the UTF-8 encoded byte string C<< $data >>.
 			if (ref($data)) {
 				$p->parse_file($data);
 			} else {
-				$p->parse_string($data);
+				if (length($data) > 0) {
+					$p->parse_string($data);
+				}
 			}
 		};
 		
 		if ($@) {
-			if ($@ =~ /no element found at line 1, column 0, byte 0/) {
+			if ($@ =~ /no element found at line 1, column 0, byte/) {
 				# silence XML::Parser output on empty input
 			} else {
 				die $@;
