@@ -7,14 +7,12 @@ AtteanX::SPARQL::Token - Token objects used for parsing and serializing SPARQL
 
 =head1 VERSION
 
-This document describes AtteanX::SPARQL::Token version 0.010
+This document describes AtteanX::SPARQL::Token version 0.011
 
 =head1 SYNOPSIS
 
   use v5.14;
   use Attean;
-  my $term = Attean::Blank->new('b1');
-  $term->ntriples_string; # _:b1
 
 =head1 DESCRIPTION
 
@@ -160,6 +158,24 @@ sub keyword {
 	return $class->fast_constructor(KEYWORD, -1, -1, -1, -1, [uc($kw)]);
 }
 
+=item C<< as_string >>
+
+Returns a string description of the token including the token type and any
+associated values.
+
+=cut
+
+sub as_string {
+	my $self	= shift;
+	my $type	= decrypt_constant($self->type);
+	my @args	= @{ $self->args };
+	if (scalar(@args)) {
+		return "$type(" . join(', ', @args) . ")";
+	} else {
+		return $type;
+	}
+}
+
 __PACKAGE__->meta->make_immutable;
 
 1;
